from abc import abstractmethod

from AnnotatedSentence.ViewLayerType import ViewLayerType
from Dictionary.Word import Word
from PropBank.ArgumentType import ArgumentType
from PropBank.Frameset import Frameset

from AnnotatedTree.ParseNodeDrawable import ParseNodeDrawable
from AnnotatedTree.ParseTreeDrawable import ParseTreeDrawable
from AnnotatedTree.Processor.Condition.IsTransferable import IsTransferable
from AnnotatedTree.Processor.NodeDrawableCollector import NodeDrawableCollector


class AutoArgument:

    secondLanguage: ViewLayerType

    @abstractmethod
    def autoDetectArgument(self, parseNode: ParseNodeDrawable, argumentType: ArgumentType) -> bool:
        pass

    def __init__(self, secondLanguage: ViewLayerType):
        self.secondLanguage = secondLanguage

    def autoArgument(self, parseTree: ParseTreeDrawable, frameset: Frameset):
        nodeDrawableCollector = NodeDrawableCollector(parseTree.getRoot(), IsTransferable(self.secondLanguage))
        leafList = nodeDrawableCollector.collect()
        for parseNode in leafList:
            if isinstance(parseNode, ParseNodeDrawable) and parseNode.getLayerData(ViewLayerType.PROPBANK) is None:
                for argumentType in ArgumentType:
                    if frameset.containsArgument(argumentType) and self.autoDetectArgument(parseNode, argumentType):
                        parseNode.getLayerInfo().setLayerData(ViewLayerType.PROPBANK,
                                                              ArgumentType.getPropbankType(argumentType))
                if Word.isPunctuationSymbol(parseNode.getLayerData(self.secondLanguage)):
                    parseNode.getLayerInfo().setLayerData(ViewLayerType.PROPBANK, "NONE")
        parseTree.save()



from FrameNet.FrameNet import FrameNet

from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from SemanticRoleLabeling.Sentence.FrameNet.SentenceAutoFramePredicate import SentenceAutoFramePredicate


class TurkishSentenceAutoFramePredicate(SentenceAutoFramePredicate):

    __frameNet: FrameNet

    def __init__(self, frameNet: FrameNet):
        """
        Constructor for TurkishSentenceAutoFramePredicate. Gets the Frames as input from the user, and sets
        the corresponding attribute.

        PARAMETERS
        ----------
        frameNet : FrameNet
            FrameNet containing the Turkish frameNet frames.
        """
        self.__frameNet = frameNet

    def autoPredicate(self, sentence: AnnotatedSentence) -> bool:
        """
        The method uses predicateFrameCandidates method to predict possible predicates. For each candidate, it sets for that
        word PREDICATE tag.

        PARAMETERS
        ----------
        sentence : AnnotatedSentence
            The sentence for which predicates will be determined automatically.

        RETURNS
        -------
        bool
            If at least one word has been tagged, true; false otherwise.
        """
        candidateList = sentence.predicateFrameCandidates(self.__frameNet)
        for word in candidateList:
            if isinstance(word, AnnotatedWord):
                word.setFrameElement("PREDICATE$NONE$" + word.getSemantic())
        if len(candidateList) > 0:
            return True
        return False
